package clus.data.type;

import java.io.IOException;

import clus.data.io.ClusReader;
import clus.data.rows.DataTuple;
import clus.data.rows.SparseDataTuple;
import clus.io.ClusSerializable;
import clus.main.Settings;
import clus.util.ClusException;

public class SparseNumericAttrType extends NumericAttrType {
	
	public final static long serialVersionUID = Settings.SERIAL_VERSION_ID;

	protected Integer m_IntIndex;
	
	public SparseNumericAttrType(String name) {
		super(name);
		setSparse(true);
	}

	public SparseNumericAttrType(NumericAttrType type) {
		super(type.getName());
		setIndex(type.getIndex());
		setSparse(true);
	}

	public SparseNumericAttrType cloneType() {
		SparseNumericAttrType at = new SparseNumericAttrType(m_Name);
		cloneType(at);
		at.setIndex(getIndex());
		return at;
	}
	
	public void setIndex(int idx) {
		m_Index = idx;
		m_IntIndex = new Integer(idx);
	}

	public int getValueType() {
		return VALUE_TYPE_NONE;
	}
	
	public float getNumeric(DataTuple tuple) {
		return ((SparseDataTuple)tuple).getfloatValueSparse(getIndex());
	}

	public boolean isMissing(DataTuple tuple) {
		return ((SparseDataTuple)tuple).getfloatValueSparse(m_IntIndex) == MISSING;
	}

	public void setNumeric(DataTuple tuple, float value) {
		((SparseDataTuple)tuple).setfloatValueSparse(value, m_IntIndex);
	}

	public ClusSerializable createRowSerializable() throws ClusException {
		return new MySerializable();
	}

	public class MySerializable extends ClusSerializable {

		public boolean read(ClusReader data, DataTuple tuple) throws IOException {
			if (!data.readNoSpace()) return false;
			float value = data.getFloat();
			setNumeric(tuple, value);
			return true;
		}
	}
}
