package clus.error;

import clus.data.attweights.ClusAttributeWeights;
import clus.data.rows.DataTuple;
import clus.data.type.NumericAttrType;
import clus.statistic.ClusStatistic;

public class RelativeError extends ClusNumericError{

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;
	protected float[] m_SumRelErr;

	public RelativeError(ClusErrorList par, NumericAttrType[] num) {
		this(par, num, null, true);
		m_SumRelErr = new float[m_Dim];
	}

	public RelativeError(ClusErrorList par, NumericAttrType[] num, ClusAttributeWeights weights) {
		this(par, num, weights, true);
		m_SumRelErr = new float[m_Dim];
	}

	public RelativeError(ClusErrorList par, NumericAttrType[] num, ClusAttributeWeights weights, boolean printall) {
		super(par, num);
		m_SumRelErr = new float[m_Dim];
	}

	@Override
	public void addExample(float[] real, float[] predicted) {
		for (int i = 0; i < m_Dim; i++) {
			float err = (real[i] - predicted[i])/real[i];
			m_SumRelErr[i] += err;

		}
	}

	public void addExample(DataTuple tuple, ClusStatistic pred) {
		float[] predicted = pred.getNumericPred();
		for (int i = 0; i < m_Dim; i++) {
			float err =(float) Math.abs(getAttr(i).getNumeric(tuple) - predicted[i])/getAttr(i).getNumeric(tuple);
			m_SumRelErr[i] += err;
		}
	}

	public void addExample(DataTuple real, DataTuple pred) {
		for (int i = 0; i < m_Dim; i++) {
				float real_i = getAttr(i).getNumeric(real);
				float predicted_i = getAttr(i).getNumeric(pred);
				float err =(float) Math.abs(real_i - predicted_i)/real_i;
				System.out.println(real_i);


				m_SumRelErr[i] += err;

		}
	}

	@Override
	public ClusError getErrorClone(ClusErrorList par) {
		// TODO Auto-generated method stub
		return null;
	}

	public float getModelErrorComponent(int i) {
		int nb = getNbExamples();
	//	System.out.println(m_SumRelErr[i]);
		float err = nb != 0.0 ? m_SumRelErr[i]/nb : 0.0f;
		System.out.println(err);

		return err;
	}

	@Override
	public String getName() {

		return "Relative Error";
	}




}
