/*************************************************************************
 * Clus - Software for Predictive Clustering                             *
 * Copyright (C) 2007                                                    *
 *    Katholieke Universiteit Leuven, Leuven, Belgium                    *
 *    Jozef Stefan Institute, Ljubljana, Slovenia                        *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                       *
 * Contact information: <http://www.cs.kuleuven.be/~dtai/clus/>.         *
 *************************************************************************/

package clus.error;

import clus.data.rows.DataTuple;
import clus.data.type.TimeSeriesAttrType;
import clus.ext.timeseries.TimeSeries;
import clus.ext.timeseries.TimeSeriesStat;
import clus.main.Settings;
import clus.statistic.ClusStatistic;

public class TSRMSError extends ClusTimeSeriesError {

	public final static long serialVersionUID = Settings.SERIAL_VERSION_ID;

	protected float m_SumSqErr;

	public TSRMSError(ClusErrorList par, TimeSeriesAttrType[] ts) {
		super(par, ts);
	}

	public void reset() {
		m_SumSqErr = 0.0f;
	}

	public void add(ClusError other) {
		TSRMSError oe = (TSRMSError)other;
		m_SumSqErr += oe.m_SumSqErr;
	}

	public void addExample(DataTuple tuple, ClusStatistic pred) {
		TimeSeries predicted = pred.getTimeSeriesPred();
		float err = ((TimeSeriesStat)pred).calcDistance(getAttr(0).getTimeSeries(tuple),  predicted);
		m_SumSqErr += sqr(err);
	}

	public final static float sqr(float value) {
		return value*value;
	}

	public float getModelErrorAdditive() {
		// return squared error not divided by the number of examples
		// optimized, e.g., by size constraint pruning
		return m_SumSqErr;
	}

	public float getModelError() {
		return getModelErrorComponent(0);
	}

	public boolean shouldBeLow() {
		return true;
	}

	public TimeSeriesAttrType getAttr(int i) {
		return m_Attrs[i];
	}

	public void addInvalid(DataTuple tuple) {
	}

	public ClusError getErrorClone(ClusErrorList par) {
		return new TSRMSError(par, m_Attrs);
	}

	public String getName() {
		return "QDMRMSError";
	}

	public float getModelErrorComponent(int i) {
		int nb = getNbExamples();
		float err = nb != 0 ? m_SumSqErr/nb : 0.0f;
		return(float) Math.sqrt(err);
	}
}
