package clus.ext.ensembles;

import java.util.ArrayList;
import java.util.Arrays;

import jeans.util.array.MDoubleArrayComparator;

import clus.data.rows.DataTuple;
import clus.ext.hierarchical.WHTDStatistic;
import clus.main.ClusStatManager;
import clus.model.ClusModel;
import clus.statistic.ClusStatistic;
import clus.statistic.RegressionStat;

public class ClusBoostingForest extends ClusForest {

	protected ArrayList m_BetaI = new ArrayList();
	protected transient MDoubleArrayComparator m_Compare = new MDoubleArrayComparator(0);
	
	public ClusBoostingForest() {
		super();
	}
	
	public ClusBoostingForest(ClusStatManager statmgr) {
		super(statmgr);
	}
	
	public void addModelToForest(ClusModel model, float beta) {
		super.addModelToForest(model);
		m_BetaI.add(new Float(beta));
	}

	public float getBetaI(int i) {
		return ((Float)m_BetaI.get(i)).floatValue();
	}
	
	public float getMedianThreshold() {
		float sum = 0.0f;
		for (int i = 0; i < m_BetaI.size(); i++) {
			sum +=(float) Math.log(1 / getBetaI(i));			
		}
		return 0.5f*sum;
	}	
	
	public ClusStatistic predictWeighted(DataTuple tuple) {
		ClusStatistic predicted = m_Stat.cloneSimple();
		predictWeightedRegression((RegressionStat)predicted, tuple);
		predicted.computePrediction();
		return predicted;
	}
	
	public void predictWeightedRegression(RegressionStat predicted,	DataTuple tuple) {
		float[] result = predicted.getNumericPred();
		float[][] treePredictions = new float[getNbModels()][];
		for (int i = 0; i < treePredictions.length; i++) {
			RegressionStat pred = (RegressionStat)getModel(i).predictWeighted(tuple);
			treePredictions[i] = pred.getNumericPred();
		}
		float medianThr = getMedianThreshold();		
		float[][] preds = new float[getNbModels()][2];
		int nbAttr = predicted.getNbAttributes();
		for (int i = 0; i < nbAttr; i++) {
			// compute weighted median of predictions of individual trees
			// weight of tree = log(1/beta)
			for (int j = 0; j < getNbModels(); j++) {
				preds[j][0] = treePredictions[j][i];
				preds[j][1] =(float) Math.log(1 / getBetaI(j));
			}
			Arrays.sort(preds, m_Compare);
			int j = 0; 
			float sum = 0.0f;
			while (true) {
				sum += preds[j][1];
				if (sum >= medianThr) break;
				j++;
			}
			result[i] = preds[j][0];			
		}
	}
	
	public ClusBoostingForest cloneBoostingForestWithThreshold(float threshold) {
		ClusBoostingForest clone = new ClusBoostingForest();
		clone.setModels(getModels());
		clone.m_BetaI = m_BetaI;
		WHTDStatistic stat = (WHTDStatistic)getStat().cloneStat();
		stat.copyAll(getStat());
		stat.setThreshold(threshold);
		clone.setStat(stat);
		return clone;
	}	
}
