/*************************************************************************
 * Clus - Software for Predictive Clustering                             *
 * Copyright (C) 2007                                                    *
 *    Katholieke Universiteit Leuven, Leuven, Belgium                    *
 *    Jozef Stefan Institute, Ljubljana, Slovenia                        *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                       *
 * Contact information: <http://www.cs.kuleuven.be/~dtai/clus/>.         *
 *************************************************************************/

package clus.ext.timeseries;

import clus.data.attweights.ClusAttributeWeights;
import clus.data.rows.DataTuple;
import clus.data.rows.RowData;
import clus.data.type.TimeSeriesAttrType;
import clus.main.ClusStatManager;
import clus.main.Settings;
import clus.statistic.ClusStatistic;
import clus.statistic.StatisticPrintInfo;

public class QDMTimeSeriesStat extends TimeSeriesStat {

	public final static long serialVersionUID = Settings.SERIAL_VERSION_ID;

	public QDMTimeSeriesStat(TimeSeriesAttrType attr) {
		super(attr);
	}
	
	public float calcDistance(TimeSeries t1, TimeSeries t2) {
		// Ljupco's measure if the time series are the same length
		// my proposal if they are not is cyclic, to be defined with Ljupco
		float[] vt1 = t1.getValuesNoCopy();
		float[] vt2 = t2.getValuesNoCopy();
		int m = Math.max(vt1.length, vt2.length);
		int n = Math.min(vt1.length, vt2.length);
		float distance = 0;
		for (int i = 0; i < m; i++) {
			for (int j = i + 1; j < m; j++) {
				//x=y means x=y
				//distance +=(float) Math.abs(Math.signum(vt1[j] - vt1[i]) -(float) Math.signum(vt2[j % n] - vt2[i % n]));
				//x=y means ABS((x/y)-1)<0.02
				distance +=(float) Math.abs(diff(vt1[j],vt1[i]) - diff(vt2[j % n],vt2[i % n]));
			}
		}
		distance = distance / (m * (m-1));
		return distance;
	}

	public static int diff(float a, float b ){
		if (a==0 && b==0) return 0;
		if (b!=0 &&(float) Math.abs((a/b)-1)<0.02)
			return 0;
		else if (a<b)
			return -1;
		else
			return 1;
	}

	public float getSS(ClusAttributeWeights scale, RowData data) {
		// TODO Auto-generated method stub
		return super.getSS(scale, data);
	}

	public void optimizePreCalc(RowData data) {
		// TODO Auto-generated method stub
		super.optimizePreCalc(data);
	}

	public ClusStatistic cloneStat() {
		QDMTimeSeriesStat stat = new QDMTimeSeriesStat(m_Attr);
		stat.cloneFrom(this);
		return stat;
	}

	public ClusStatistic cloneSimple() {
		QDMTimeSeriesStat stat = new QDMTimeSeriesStat(m_Attr);
		stat.m_RepresentativeMean = new TimeSeries(m_RepresentativeMean.length());
		stat.m_RepresentativeMedian = new TimeSeries(m_RepresentativeMedian.length());
		return stat;
	}

	public void copy(ClusStatistic other) {
		QDMTimeSeriesStat or = (QDMTimeSeriesStat) other;
		super.copy(or);
		m_Value = or.m_Value;
	}

	public float getError(ClusAttributeWeights scale) {
		// TODO Auto-generated method stub
		return Float.POSITIVE_INFINITY;
	}

	/*
	 * [Aco]
	 * for printing in the nodes
	 * @see clus.statistic.ClusStatistic#getString(clus.statistic.StatisticPrintInfo)
	 */
	public String getString(StatisticPrintInfo info){
		return super.getString(info);
	}

	/*
	 * [Aco]
	 * a new timeseries comes, and we calculate something for it
	 * @see clus.statistic.ClusStatistic#updateWeighted(clus.data.rows.DataTuple, int)
	 */
	public void updateWeighted(DataTuple tuple, int idx){
		//System.out.println("noproto");
	    super.updateWeighted(tuple,idx);
	}

	/*
	 * [Aco]
	 * this is executed in the end
	 * @see clus.statistic.ClusStatistic#calcMean()
	 */
	public void calcMean() {
		super.calcMean();
	}


}
